<?php
/**
 * GMapFP! Openstreet Field Project
 * Version J3_1
 * Creation date: Novembre 2019
 * Author: Fabrice4821 - https://creation-web.pro
 * Author email: webmaster@gmapfp.org
 *
 * @copyright   Copyright (C) 2011 - 2019 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

JLoader::import('components.com_fields.libraries.fieldsplugin', JPATH_ADMINISTRATOR);

use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\Registry\Registry;

class PlgFieldsGmapFP_Openstreet extends FieldsPlugin
{
	public function onCustomFieldsPrepareDom($field, DOMElement $parent, JForm $form)
	{
		$fieldNode = parent::onCustomFieldsPrepareDom($field, $parent, $form);

		if (!$fieldNode)
		{
			return $fieldNode;
		}

		$doc = Factory::getDocument();
		$field->name_ = str_replace('-', '_', $field->name);
		$targetid = 'jform_com_fields_' . $field->name_;

		$plugin = PluginHelper::getPlugin('fields', 'gmapfp_openstreet');
		
		if(empty($plugin) or !property_exists($plugin, 'params')) {
			echo Text::_('PLG_FIELDS_GMAPFP_OPENSTREET_NON_CONFIGURE_OR_ACTIF');
		} else {
			$params = new Registry($plugin->params);
				
			$lat = '999999';
			$datas = explode(',', $field->rawvalue);
			if (count($datas)>3) {
				$lat = $datas[0];
				$lng = $datas[1];
				$zoom = $datas[2];
				$lenth = strlen ($lat.$lng.$zoom)+3;
				$texte = str_replace('"', '\"', substr($field->rawvalue, $lenth));
			} elseif ($field->default_value) {
				$datas = explode(',', $field->default_value);
				if (count($datas)>3) {
					$lat = $datas[0];
					$lng = $datas[1];
					$zoom = $datas[2];
					$lenth = strlen ($lat.$lng.$zoom)+3;
					$texte = substr($field->rawvalue, $lenth);
				}
			} 
			if($lat == '999999') {
				$lat = 47.91474210103591;
				$lng = 2.146266102790833;
				$zoom = 7;
				$texte = "";
			}

			$layers = '';
			$baseLayers = array();
			$aliasLayers = array();
			$paramsLayers = $params->get('gmapfp_osm_layers');
			$defaultLayer = '';
			if ($paramsLayers)
				foreach($paramsLayers AS $layer){
					$layers .= 'var '.$layer->alias.' = L.tileLayer("'.$layer->url.'", {';
					$layers .= 'attribution: "'.str_replace(array('"','‹'), array('\"','<'), $layer->attribution).'",';
					$layers .= 'minZoom: 1,';
					$layers .= 'maxZoom: '.$layer->max_zoom.',';
					$layers .= 'id: "'.$layer->alias.'"';
					$layers .= '});';
					$layers .= 'mes_layers'.$field->name_.'["'.$layer->alias.'"] = '.$layer->alias.';';
					$baseLayers[] = '"'.$layer->name.'":'.$layer->alias;
					$aliasLayers[] = $layer->alias;
				}
			$var_baseLayers = 'baseLayers'.$field->name_.' = {'.implode(',', $baseLayers).'};';
			
			if (in_array($params->get('gmapfp_type_admin_osm'), $aliasLayers))
				$defaultLayer = '';
			elseif (array_key_exists(0, $aliasLayers))
				$defaultLayer = 'stylemap = "'.$aliasLayers[0].'";';
			
			HTMLHelper::_('stylesheet', 'https://unpkg.com/leaflet/dist/leaflet.css');
			HTMLHelper::_('script', 'https://unpkg.com/leaflet/dist/leaflet-src.js');
			
			HTMLHelper::_('script', 'https://unpkg.com/esri-leaflet');
			HTMLHelper::_('script', 'https://unpkg.com/esri-leaflet-geocoder');
			HTMLHelper::_('stylesheet', 'https://unpkg.com/esri-leaflet-geocoder/dist/esri-leaflet-geocoder.css');
			

			$js = ''
				.'var map'.$field->name_.';'
				.'var marker1'.$field->name_.';'
				.'var mes_layers'.$field->name_.' = [];'
				.'var lat'.$field->name_.' = '.$lat.';'
				.'var lng'.$field->name_.' = '.$lng.';'
				.'var zoom'.$field->name_.' = '.$zoom.';'
				.'var text'.$field->name_.' = "'.$texte.'";'
				.'var searchControl = "";'
				.'var baseLayers'.$field->name_.' = "";'
				.''
				.''
				.''
				.'var results = "";'
				.''
				.'function init_osm() {'
				.'	var stylemap;'
				.''
				.$defaultLayer
				.''
				.$layers
				.''
				.'	var options = {'
				.'		center: [lat'.$field->name_.', lng'.$field->name_.'],'
				.'		zoom: zoom'.$field->name_.','
				.'		layers: [mes_layers'.$field->name_.'[stylemap]]'
				.'	};'
				.''
				.'	map'.$field->name_.' = L.map("map'.$targetid.'", options);'
				.''
				//affichce l'échelle
				.'	L.control.scale().addTo(map'.$field->name_.');'
				.''
				.$var_baseLayers
				.''
				.'	L.control.layers(baseLayers'.$field->name_.').addTo(map'.$field->name_.');'
				.''
				// create the geocoding control and add it to the map
				.'	searchControl = L.esri.Geocoding.geosearch().addTo(map'.$field->name_.');'
				.'	results = L.layerGroup().addTo(map'.$field->name_.');'
				.''
				// listen for the results event and add every result to the map
				.'	searchControl.on("results", function(data) {'
				.'		results.clearLayers();'
				.'		marker1'.$field->name_.'.setLatLng(data.results[0].latlng);'
				.'		lat'.$field->name_.' = data.results[0].latlng.lat;'
				.'		lng'.$field->name_.' = data.results[0].latlng.lng;'
				.'		jQuery("#lat_'.$targetid.'").val(lat'.$field->name_.');'
				.'		jQuery("#lng_'.$targetid.'").val(lng'.$field->name_.');'
				.'		updateresult'.$field->name_.'();'
				.'	});'
				.''
				// Create a draggable marker which will later on be binded to a'
				.'	marker1'.$field->name_.' = L.marker([lat'.$field->name_.', lng'.$field->name_.'], {draggable: true}).addTo(map'.$field->name_.');'
				.'	marker1'.$field->name_.'.bindPopup("'.Text::_('PLG_FIELDS_GMAPFP_OPENSTREET_DRAG_ME').'");'."\r\n"
				.''
				.'	marker1'.$field->name_.'.on("dragend", function(ev){'
				.'		var position = ev.target;'
				.'		lat'.$field->name_.' = position.getLatLng().lat;'
				.'		lng'.$field->name_.' = position.getLatLng().lng;'
				.'		jQuery("#lat_'.$targetid.'").val(lat'.$field->name_.');'
				.'		jQuery("#lng_'.$targetid.'").val(lng'.$field->name_.');'
				.'		updateresult'.$field->name_.'();'
				.'	});'
				.''
				.'	map'.$field->name_.'.on("zoom", function(ev){'
				.'		var zoom = ev.target;'
				.'		zoom'.$field->name_.' = zoom.getZoom();'
				.'		updateresult'.$field->name_.'();'
				.'	});'
				.''
				.'}'
				.''
				// Register an event listener to fire when the page finishes loading
				.'jQuery( document ).ready(initialise_map_gmapfp'.$field->name_.');'
				.''
				.'var tstGMapFP'.$field->name_.';'
				.'var tstIntGMapFP'.$field->name_.';'
				.''
				.'function CheckGMapFP'.$field->name_.'() {'
				.'	if (tstGMapFP'.$field->name_.') {'
				.'		if (tstGMapFP'.$field->name_.'.offsetWidth != tstGMapFP'.$field->name_.'.getAttribute("oldValue")) {'
				.'			tstGMapFP'.$field->name_.'.setAttribute("oldValue",tstGMapFP'.$field->name_.'.offsetWidth);'
				.'			init_osm();'
				.'		}'
				.'	}'
				.'}'
				.''
				.'function updateresult'.$field->name_.'() {'
				.'	jQuery("#'.$targetid.'").val(lat'.$field->name_.'+","+lng'.$field->name_.'+","+zoom'.$field->name_.'+","+text'.$field->name_.');'
				.'}'
				.''
				.'function lat_'.$field->name_.'(lat) {'
				.'	marker1'.$field->name_.'.setLatLng([lat,lng'.$field->name_.']);'
				.''
				.'}'
				.''
				.'function initialise_map_gmapfp'.$field->name_.'() {'
				// Créer la div map
				// .'	jQuery("#'.$targetid.'").css("display", "none");'
				.'	jQuery("#'.$targetid.'").parent().prepend("<label for=\'text_'.$targetid.'\'>'.Text::_('PLG_FIELDS_GMAPFP_OPENSTREET_MARKER_TEXT').'</label><textarea onchange=\'text'.$field->name_.'=this.value;updateresult'.$field->name_.'();\' type=\'text\' id=\'text_'.$targetid.'\'>'.$texte.'</textarea>");'
				.'	jQuery("#'.$targetid.'").parent().prepend("<label for=\'lng_'.$targetid.'\'>'.Text::_('PLG_FIELDS_GMAPFP_OPENSTREET_LNG').'</label><input onchange=\'lng_'.$field->name_.'(this.value);updateresult'.$field->name_.'();\' type=\'text\' id=\'lng_'.$targetid.'\' value=\''.$lng.'\'/>");'
				.'	jQuery("#'.$targetid.'").parent().prepend("<label for=\'lat_'.$targetid.'\'>'.Text::_('PLG_FIELDS_GMAPFP_OPENSTREET_LAT').'</label><input onchange=\'lat_'.$field->name_.'(this.value);updateresult'.$field->name_.'();\' type=\'text\' id=\'lat_'.$targetid.'\' value=\''.$lat.'\'/>");'
				.'	jQuery("#'.$targetid.'").parent().prepend("<div style=\'height:300px;\' id=\'map'.$targetid.'\'></div>");'
				.'	jQuery("#'.$targetid.'").css("display", "none");'
				.'	tstGMapFP'.$field->name_.' = document.getElementById("map'.$targetid.'");'
				.'  tstGMapFP'.$field->name_.'.setAttribute("oldValue",0);'
				.'  tstIntGMapFP'.$field->name_.' = setInterval("CheckGMapFP'.$field->name_.'()",500);'
				.'}'
				.''
				;

			$doc->addScriptDeclaration($js);

		}

		// Set filter to user UTC
		$fieldNode->setAttribute('filter', 'raw');

		// Set field to use translated formats
		$fieldNode->setAttribute('translateformat', '0');

		return $fieldNode;
	}
}
